@@ -19,6 +19,85 @@ __all__ = (
 )
 
 
+async def _get_top_matchmaking(page: int = 0, royal: bool = False):
+    _log.debug(f"Getting top matchmaking players page {page}. Royal? {royal}")
+
+    cache_client = Client._get_cache_client()
+
+    with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
+        if cache_client.exists(f"top_matchmaking:{page}:{royal}"):
+            _log.debug(f"Found top matchmaking players for page {page} in cache")
+            return json.loads(
+                cache_client.get(f"top_matchmaking:{page}:{royal}").decode("utf-8")
+            ).get("ranks")
+    api_client = _APIClient()
+
+    if not royal:
+        match_history = await api_client.get(
+            _TMIO.build([_TMIO.TABS.TOP_MATCHMAKING, str(page)])
+        )
+    else:
+        match_history = await api_client.get(
+            _TMIO.build([_TMIO.TABS.TOP_ROYAL, str(page)])
+        )
+
+    await api_client.close()
+
+    with suppress(KeyError, TypeError):
+        raise TMIOException(match_history["error"])
+    with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
+        _log.debug(f"Caching top matchmaking players for page {page}")
+        cache_client.set(
+            f"top_matchmaking:{page}:{royal}", json.dumps(match_history), ex=3600
+        )
+
+    return match_history.get("ranks")
+
+
+async def _get_history(player_id: str, type_id: int, page: int) -> List[Dict]:
+    if player_id is None:
+        raise InvalidIDError("Player ID is not set.")
+
+    _log.debug("Getting matchmaking history for player %s and page %d", player_id, page)
+
+    cache_client = Client._get_cache_client()
+
+    with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
+        if cache_client.exists(f"mm_hist:{page}:{type_id}:{player_id}"):
+            _log.debug("Found matchmaking history for page %s in cache", page)
+            return json.loads(
+                cache_client.get(f"mm_hist:{page}:{type_id}:{player_id}").decode(
+                    "utf-8"
+                )
+            )["history"]
+
+    api_client = _APIClient()
+    match_history = await api_client.get(
+        _TMIO.build(
+            [
+                _TMIO.TABS.PLAYER,
+                player_id,
+                _TMIO.TABS.MATCHES,
+                type_id,
+                page,
+            ]
+        )
+    )
+    await api_client.close()
+
+    with suppress(KeyError, TypeError):
+        raise TMIOException(match_history["error"])
+    with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
+        _log.debug(f"Saving matchmaking history for page {page} to cache")
+        cache_client.set(
+            f"mm_history:{page}:{type_id}:{player_id}",
+            json.dumps(match_history),
+            ex=3600,
+        )
+
+    return match_history.get("history", [])
+
+
 class PlayerMatchmakingResult:
     """
     .. versionadded :: 0.3.0
@@ -35,7 +114,7 @@ class PlayerMatchmakingResult:
         The live id of the match
     mvp : bool
         Whether the player was the mvp of the match
-    player_id : str
+    player_id : str | None
         The player's ID
     start_time : :class:`datetime`
         The date the match started
@@ -65,14 +144,16 @@ class PlayerMatchmakingResult:
     def _from_dict(cls, data: Dict, player_id: str = None):
         _log.debug("Creating a PlayerMatchmakingResult class from given dictionary")
 
-        after_score = data["afterscore"]
-        leave = data["leave"]
-        live_id = data["lid"]
-        mvp = data["mvp"]
-        start_time = datetime.strptime(data["starttime"], "%Y-%m-%dT%H:%M:%SZ")
-        win = data["win"]
+        after_score = data.get("afterscore")
+        leave = data.get("leave")
+        live_id = data.get("lid")
+        mvp = data.get("mvp")
+        start_time = datetime.strptime(data.get("startime"), "%Y-%m-%dT%H:%M:%SZ")
+        win = data.get("win")
 
-        return cls(after_score, leave, live_id, mvp, player_id, start_time, win)
+        args = [after_score, leave, live_id, mvp, player_id, start_time, win]
+
+        return cls(*args)
 
 
 class PlayerMatchmaking:
@@ -83,7 +164,7 @@ class PlayerMatchmaking:
 
     Parameters
     ----------
-    type : str
+    matchmaking_type : str
         The type of matchmaking, either "3v3" or "Royal"
     type_id : int
         The type of matchmaking as 0 or 1, 0 for "3v3" and 1 for "Royal"
@@ -97,10 +178,12 @@ class PlayerMatchmaking:
         The division of the player in matchmaking
     division_str : str: str
         The division of the player in matchmaking as a string
-    min_points : ints: int
+    min_points : int
         The points required to reach the current division.
-    max_points : ints: int
+    max_points : int
         The points required to move up the rank.
+    player_id : str | None
+        The player's ID. Defaults to None
     """
 
     def __init__(
@@ -113,7 +196,7 @@ class PlayerMatchmaking:
         division: int,
         min_points: int,
         max_points: int,
-        player_id: str = None,
+        player_id: str | None = None,
     ):
         """Constructor for the class."""
         MATCHMAKING_STRING = {
@@ -138,7 +221,7 @@ class PlayerMatchmaking:
         self.score = score
         self.progression = progression
         self.division = division
-        self.division_str = MATCHMAKING_STRING[division]
+        self.division_str = MATCHMAKING_STRING.get(division)
         self._min_points = min_points
         self._max_points = 1 if max_points == 0 else max_points
         self.player_id = player_id
@@ -174,8 +257,9 @@ class PlayerMatchmaking:
         if len(mm_data) == 0:
             matchmaking_data.extend([None, None])
         elif len(mm_data) == 1:
+            mm_obj = PlayerMatchmaking.__parse_3v3(mm_data[0])
             matchmaking_data.extend(
-                [PlayerMatchmaking.__parse_3v3(mm_data[0], player_id), None]
+                [mm_obj, None] if mm_obj.type_id == 2 else [None, mm_obj]
             )
         else:
             matchmaking_data.extend(
@@ -206,27 +290,20 @@ class PlayerMatchmaking:
         )
 
         if "info" in data:
-            typename = data["info"]["typename"]
-            typeid = data["info"]["typeid"]
-            progression = data["info"]["progression"]
-            rank = data["info"]["rank"]
-            score = data["info"]["score"]
-            division = data["info"]["division"]["position"]
-            min_points = data["info"]["division"]["minpoints"]
-            max_points = data["info"]["division"]["maxpoints"]
-        else:
-            typename = data["typename"]
-            typeid = data["typeid"]
-            progression = data["progression"]
-            rank = data["rank"]
-            score = data["score"]
-            division = data["division"]["position"]
-            min_points = data["division"]["minpoints"]
-            max_points = data["division"]["maxpoints"]
+            data = data.get("info")
 
-        return cls(
-            typename,
-            typeid,
+        type_name = data.get("typename")
+        type_id = data.get("typeid")
+        progression = data.get("progression")
+        rank = data.get("rank")
+        score = data.get("score")
+        division = data.get("division").get("position")
+        min_points = data.get("division").get("minpoints")
+        max_points = data.get("division").get("maxpoints")
+
+        args = [
+            type_name,
+            type_id,
             progression,
             rank,
             score,
@@ -234,21 +311,36 @@ class PlayerMatchmaking:
             min_points,
             max_points,
             player_id,
-        )
+        ]
+
+        return cls(*args)
 
     @property
     def min_points(self):
-        """min points property"""
+        """min points"""
         return self._min_points
 
     @property
     def max_points(self):
-        """max points property"""
+        """max points"""
         return self._max_points
 
+    def __str__(self):
+        progression = self.progression
+        progress = self.progress
+        rank = self.rank
+        score = self.score
+        division = self.division
+        division_str = self.division_str
+        max_points = self.max_points
+
+        return f"Progression: {progression}\nProgress: {progress}\nRank: {rank}\nScore: {score}\nDivision: {division_str} - {division}\n\nPoints to Next Division: {max_points + 1}"
+
     async def history(self, page: int = 0) -> List[PlayerMatchmakingResult]:
         """
         .. versionadded :: 0.3.0
+        .. versionchanged :: 0.4.0
+            Use `_get_history()` helper command.
 
         History of recent matches in this matchmaking
 
@@ -267,63 +359,15 @@ class PlayerMatchmaking:
         :class:`InvalidIDError`
             If the player_id is not set.
         """
-        if self.player_id is None:
-            raise InvalidIDError("Player ID is not set")
+        matches = await _get_history(self.player_id, self.type_id, page)
 
-        _log.debug(
-            f"Getting matchmaking history page {page} for player {self.player_id}"
-        )
-
-        cache_client = Client._get_cache_client()
-
-        with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
-            if cache_client.exists(
-                f"mm_history:{page}:{self.type_id}:{self.player_id}"
-            ):
-                _log.debug(f"Found matchmaking history for page {page} in cache")
-
-                player_results = []
-                history = json.loads(
-                    cache_client.get(
-                        f"mm_history:{page}:{self.type_id}:{self.player_id}"
-                    ).decode("utf-8")
-                )
-
-                for item in history["matches"]:
-                    player_results.append(PlayerMatchmakingResult._from_dict(item))
-
-                return player_results
-
-        api_client = _APIClient()
-        match_history = await api_client.get(
-            _TMIO.build(
-                [
-                    _TMIO.TABS.PLAYER,
-                    self.player_id,
-                    _TMIO.TABS.MATCHES,
-                    self.type_id,
-                    str(page),
-                ]
-            )
-        )
-        await api_client.close()
-
-        with suppress(KeyError, TypeError):
-
-            raise TMIOException(match_history["error"])
-        with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
-            _log.debug(f"Saving matchmaking history for page {page} to cache")
-            cache_client.set(
-                f"mm_history:{page}:{self.type_id}:{self.player_id}",
-                json.dumps(match_history),
-                ex=3600,
+        match_results = []
+        for match in matches:
+            match_results.append(
+                PlayerMatchmakingResult._from_dict(match), self.player_id
             )
 
-        player_results = []
-        for match in match_history["matches"]:
-            player_results.append(PlayerMatchmakingResult._from_dict(match))
-
-        return player_results
+        return match_results
 
     @staticmethod
     async def top_matchmaking(page: int = 0, royal: bool = False) -> List[Dict]:
@@ -344,36 +388,4 @@ class PlayerMatchmaking:
         :class:`List[Dict]`
             The top matchmaking players by score. Each page contains 50 players.
         """
-        _log.debug(f"Getting top matchmaking players page {page}. Royal? {royal}")
-
-        cache_client = Client._get_cache_client()
-
-        with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
-            if cache_client.exists(f"top_matchmaking:{page}:{royal}"):
-                _log.debug(f"Found top matchmaking players for page {page} in cache")
-                return json.loads(
-                    cache_client.get(f"top_matchmaking:{page}:{royal}").decode("utf-8")
-                )["ranks"]
-        api_client = _APIClient()
-
-        if not royal:
-            match_history = await api_client.get(
-                _TMIO.build([_TMIO.TABS.TOP_MATCHMAKING, str(page)])
-            )
-        else:
-            match_history = await api_client.get(
-                _TMIO.build([_TMIO.TABS.TOP_ROYAL, str(page)])
-            )
-
-        await api_client.close()
-
-        with suppress(KeyError, TypeError):
-
-            raise TMIOException(match_history["error"])
-        with suppress(ConnectionRefusedError, redis.exceptions.ConnectionError):
-            _log.debug(f"Caching top matchmaking players for page {page}")
-            cache_client.set(
-                f"top_matchmaking:{page}:{royal}", json.dumps(match_history), ex=3600
-            )
-
-        return match_history["ranks"]
+        return await _get_top_matchmaking(page, royal)
